import java.util.Comparator;

import com.atlassian.jira.issue.customfields.searchers.transformer.CustomFieldInputHelper;
import com.atlassian.jira.issue.fields.CustomField;
import com.atlassian.jira.issue.search.SearchRequest;
import com.atlassian.jira.issue.statistics.StatisticsMapper;
import com.atlassian.jira.jql.builder.JqlClauseBuilder;
import com.atlassian.jira.jql.builder.JqlQueryBuilder;
import com.atlassian.jira.security.JiraAuthenticationContext;

public class TextStatisticsMapper implements StatisticsMapper
{
    private CustomField customField;
	private CustomFieldInputHelper customFieldInputHelper;
	private JiraAuthenticationContext authenticationContext;
	private boolean isExact = true;

	public TextStatisticsMapper(CustomField customField, CustomFieldInputHelper customFieldInputHelper,
			final JiraAuthenticationContext authenticationContext) {
		this.customField = customField;
		this.customFieldInputHelper = customFieldInputHelper;
		this.authenticationContext = authenticationContext;
	}

	@Override
	public Comparator getComparator() {
		// FIXME really case insensitive?
		return String.CASE_INSENSITIVE_ORDER;
	}

	@Override
	public String getDocumentConstant() {
		return customField.getId();
	}

	// to be used in query
	protected String getClauseName() {
		return customFieldInputHelper.getUniqueClauseName(authenticationContext.getLoggedInUser(), customField.getClauseNames()
				.getPrimaryName(), customField.getName());
	}

	@Override
	public Object getValueFromLuceneField(String documentValue) {
		// for example in ProjectSearcher only project key is stored in
		// customfield and in this method it should be converted to
		// ProjectObject
		// in our case the stored value (String) is equal represented object
		return documentValue;
	}

	@Override
	public SearchRequest getSearchUrlSuffix(Object value, SearchRequest searchRequest) {

		if (searchRequest == null) {
			return null;
		} else {
			String str_value = this.isValidValue(value) ? (String) value : null;
			JqlClauseBuilder builder = JqlQueryBuilder.newBuilder(searchRequest.getQuery()).where().defaultAnd();

			if (str_value != null) {
				if (isExact) {
					builder.customField(customField.getIdAsLong()).eq(str_value);
				}
				else {
					builder.customField(customField.getIdAsLong()).like(str_value);
				}
			} else {
				builder.addEmptyCondition(getClauseName());
			}
			return new SearchRequest(builder.buildQuery());
		}
	}

	@Override
	public boolean isFieldAlwaysPartOfAnIssue() {
		return false;
	}

	@Override

	public boolean isValidValue(Object value) {
		// TODO (optional) includeEmpty as in com.atlassian.jira.issue.statistics.LabelsStatisticsMapper
		return value != null;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}

		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		final TextStatisticsMapper that = (TextStatisticsMapper) o;

		return (getDocumentConstant() != null ? getDocumentConstant().equals(that.getDocumentConstant()) : that
				.getDocumentConstant() == null);
	}

	@Override
	public int hashCode() {
		return (getDocumentConstant() != null ? getDocumentConstant().hashCode() : 0);
	}

	public boolean isExact() {
		return isExact;
	}

	public void setExact(boolean isExact) {
		this.isExact = isExact;
	}
}
